@@ -1,4 +1,5 @@
-from django.conf.urls import patterns, include, url
+from django.conf.urls import include, url
+from django.contrib.staticfiles.urls import staticfiles_urlpatterns
 from django.conf.urls.static import static
 from django.conf import settings
 
@@ -8,7 +9,7 @@ admin.autodiscover()
 
 import app
 
-urlpatterns = patterns('',
+urlpatterns = [
     # Examples:
     # url(r'^$', 'example_project.views.home', name='home'),
     # url(r'^example_project/', include('example_project.foo.urls')),
@@ -17,7 +18,9 @@ urlpatterns = patterns('',
     # url(r'^admin/doc/', include('django.contrib.admindocs.urls')),
 
     # Uncomment the next line to enable the admin:
-    url(r'^admin/', include(admin.site.urls)),
+    url(r'^admin/', admin.site.urls),
 
     url(r'', include('app.urls')),
-) + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
+] \
++ staticfiles_urlpatterns() \
++ static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
